/*
 * This tests a flow for cloudron owner creating a cloudron
 * with a custom domain from the appstore. Owner creates a cloudron, activates
 * it, installs and app and deletes the cloudron eventually
 */

'use strict';

var AppStore = require('../appstore.js'),
    assert = require('assert'),
    async = require('async'),
    AWS = require('aws-sdk'),
    Cloudron = require('../cloudron.js'),
    common = require('../common.js'),
    dns = require('dns'),
    request = require('superagent-sync');

require('colors');

process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';
var BOX_VERSION = process.env.BOX_VERSION;
var CUSTOM_DOMAIN = process.env.CUSTOM_DOMAIN;

describe('Custom domain test', function () {
    this.timeout(0);

    var appStore = new AppStore('https://api.staging.cloudron.io');
    var route53 = new AWS.Route53({ accessKeyId: process.env.AWS_STAGING_ACCESS_KEY, secretAccessKey: process.env.AWS_STAGING_SECRET_KEY });

    var owner = common.getOwner();
    var admin = common.getAdmin();
    var cloudron, appId, box, backupInfo;

    it('can login to the store', function () {
        var accessToken = appStore.getAccessToken(owner);
        appStore.setCredentials(owner.password, accessToken);

        var adminAccessToken = appStore.getAccessToken(admin);
        appStore.setAdminCredentials(admin.password, adminAccessToken);
    });

    it('can get profile', function () {
        var profile = appStore.getProfile();
        owner.id = profile.id;
    });

    it('can setup billing details', function (done) {
        appStore.setupBilling(owner, done);
    });

    it('can create a cloudron', function () {
        box = appStore.createCloudron({
            domain: CUSTOM_DOMAIN,
            region: 'ams3',
            size: '1gb',
            version: BOX_VERSION
        });
    });

    it('can setup my. domain after it got IP', function (done) {
        var ip = appStore.waitForIP(box.id);

        // update the route53 record for my subdomain
        route53.listHostedZonesByName({ DNSName: CUSTOM_DOMAIN, MaxItems: '1'}, function (error, result) {
            if (error) return done(error);

            var params = {
                ChangeBatch: {
                    Changes: [{
                        Action: 'UPSERT',
                        ResourceRecordSet: {
                            Type: 'A',
                            Name: 'my.' + CUSTOM_DOMAIN + '.',
                            ResourceRecords: [ { Value: ip } ],
                            TTL: 1
                        }
                    }]
                },
                HostedZoneId: result.HostedZones[0].Id
            };

            route53.changeResourceRecordSets(params, function(error) {
                if (error) return done(error);

                done();
            });
        });
    });

    it('can wait for cloudron', function () {
        box = appStore.waitForCloudron(box.id);
        cloudron = new Cloudron(box);
    });

    it('can activate the box', function () {
        cloudron.activate(owner);
    });

    it('can login to the box', function () {
        var token = cloudron.getOauthToken(owner);
        cloudron.setCredentials(owner.password, token);
    });

    it('can set dns credentials', function () {
        cloudron.setDnsConfig({ provider: 'route53', accessKeyId: process.env.AWS_STAGING_ACCESS_KEY, secretAccessKey: process.env.AWS_STAGING_SECRET_KEY });
    });
    var location = 'test' + (Math.random() * 10000).toFixed();
    it('can install app', function () {
        var manifest = appStore.getManifest(common.TESTAPP_ID, common.TESTAPP_VERSION);
        appId = cloudron.installApp(location, manifest);
    });

    it('can populate the addons', function () {
        cloudron.populateAddons(cloudron.appFqdn(location));
    });

    it('can check the addons', function () {
        cloudron.checkAddons(cloudron.appFqdn(location), owner);
    });

    it('can backup the box', function () {
        backupInfo = cloudron.backup();
        assert.strictEqual(backupInfo.dependsOn.length, 1);
    });

    it('can restore the box', function () {
        appStore.restore(box.id, backupInfo.id);
        box = appStore.waitForCloudron(box.id);
    });

    it('wait for app to be ready', function () {
        cloudron.waitForApp(appId);
    });

    it('can configure app', function () {
        location = location + 'x';
        cloudron.configureApp(appId, location);
    });

    it('can check the addons', function () {
        cloudron.checkAddons(cloudron.appFqdn(location), owner);
    });

    it('can reboot the cloudron', function () {
        cloudron.reboot();
    });

    it('can check the addons', function () {
        cloudron.checkAddons(cloudron.appFqdn(location), owner);
    });

    it('can uninstall app', function () {
        cloudron.uninstallApp(appId);
    });

    // check this after activation
    it('has setup DNS records correctly', function (done) {
        async.series([
            // cloudron.checkA.bind(cloudron), // this is at user's discretion
            cloudron.checkSPF.bind(cloudron),
            cloudron.checkDKIM.bind(cloudron),
            cloudron.checkMX.bind(cloudron),
            // cloudron.checkDMARC.bind(cloudron)
        ], done);
    });

    it('can delete the cloudron', function () {
        appStore.deleteCloudron(box);
    });
});
